/*
 * Project name:
      OLED W Click (SPI)
 * Copyright:
      (c) mikroElektronika, 2014.
 * Revision History:
      20140918:
      - Initial release (BD);
 * Description:
      OLED W click carries a 96 x 39px blue monochrome passive matrix OLED display.
      The display is bright, has a wide viewing angle and low power consumption.
      To drive the display, OLED W click features an SSD1306 controller.
      It's built-in functionalities include contrast control, normal or inverse image display,
      vertical and horizontal scrolling functions and more.
      OLED W click can communicate with the target board MCU either through SPI or I2C mikroBUS lines.
      You switch between output options by resoldering the onboard SEL COMM jumpers (J1, J2 and J3) to the appropriate position.
      OLED W click uses a 3.3V power supply.

      Description of some terms: OLED M - OLED Monochrome module, OLED B - OLED Blue, OLED W - OLED White, OLED C - OLED Color

 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
     ext. modules:    OLED W click
                      http://www.mikroe.com/click/oled-w/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 NOTES:
     - Place OLED W Click board in the mikroBUS socket 1.
     - This code example is for SPI protocol. Set SEL COMM jumpers (J1, J2 and J3) to the appropriate position for SPI.
*/

#include "Oled_M.h"

sbit OLED_RST at GPIOC_ODR.B3;
sbit OLED_CS at GPIOD_ODR.B13;
sbit OLED_DC at GPIOA_ODR.B0;

unsigned char pic[]={
0xFF, 0xFF, 0x3F, 0x1F, 0x0F, 0x07, 0x07, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0xE3, 0xE3,
0x03, 0x03, 0x03, 0x03, 0xC3, 0xE3, 0xC3, 0x03, 0x03, 0x03, 0x03, 0xE3, 0xE3, 0x03, 0x03, 0x03,
0x03, 0x03, 0x03, 0x03, 0x03, 0xC3, 0xC3, 0x63, 0x63, 0x63, 0x63, 0x63, 0xE3, 0xE3, 0x03, 0x03,
0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0xE3, 0xE3, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
0x03, 0xE3, 0xE3, 0x03, 0x03, 0x03, 0xC3, 0xC3, 0x63, 0x63, 0x63, 0x63, 0xC3, 0xC3, 0x03, 0x03,
0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x07, 0x07, 0x0F, 0x1F, 0x3F, 0xFF, 0xFF,
0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03,
0x1F, 0xF8, 0xC0, 0xFE, 0x1F, 0x00, 0x1F, 0xFE, 0xC0, 0xF8, 0x1F, 0x03, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x3F, 0xFF, 0xC0, 0x80, 0x80, 0x80, 0x80, 0x80, 0xFF, 0xFF, 0x00, 0x00,
0x8C, 0x8C, 0x8C, 0x8C, 0x8C, 0x8C, 0x8C, 0xFF, 0xFF, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80,
0x80, 0xFF, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0xC0, 0x80, 0x80, 0x80, 0x80, 0xC0, 0xFF, 0x3F, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF,
0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
0x80, 0x80, 0x81, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x81, 0x81, 0x01, 0x01, 0x80, 0x80,
0x01, 0x01, 0xC1, 0xC1, 0xC1, 0x81, 0x01, 0x01, 0x01, 0x00, 0x00, 0x81, 0xC1, 0xC1, 0xC1, 0x01,
0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF,
0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x31, 0x31, 0x31, 0x31, 0x31, 0x31,
0x31, 0xFF, 0xFF, 0x00, 0x00, 0xF0, 0xF8, 0x1C, 0x0C, 0x0C, 0x1C, 0xF8, 0xF0, 0x00, 0x0C, 0x0C,
0x18, 0xFC, 0xFC, 0x00, 0x00, 0x04, 0x8C, 0xF8, 0x70, 0xE0, 0xFF, 0xFF, 0x00, 0x00, 0xFD, 0xFD,
0x00, 0x00, 0xFF, 0xFF, 0x03, 0x1F, 0xFC, 0xC0, 0x00, 0xC0, 0xFC, 0x1F, 0x03, 0xFF, 0xFF, 0x00,
0x00, 0x00, 0x00, 0x00, 0x0C, 0x3C, 0xF0, 0xC0, 0x00, 0xC0, 0xF0, 0x3C, 0x0C, 0x00, 0xF0, 0xF8,
0x1C, 0x0C, 0x0C, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF,
0xFF, 0xFE, 0xF8, 0xF0, 0xE0, 0xC0, 0xC0, 0x80, 0x80, 0x80, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86,
0x86, 0x87, 0x87, 0x80, 0x80, 0x81, 0x83, 0x87, 0x86, 0x86, 0x87, 0x83, 0x81, 0x80, 0x80, 0x80,
0x80, 0x87, 0x87, 0x80, 0x80, 0x86, 0x87, 0x81, 0x80, 0x80, 0x87, 0x87, 0x80, 0x80, 0x87, 0x87,
0x80, 0x80, 0x87, 0x87, 0x80, 0x80, 0x81, 0x87, 0x86, 0x87, 0x81, 0x80, 0x80, 0x87, 0x87, 0x80,
0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x81, 0x87, 0x9F, 0xBF, 0xB1, 0xB0, 0x80, 0x80, 0x81, 0x83,
0x87, 0x86, 0x86, 0x83, 0x87, 0x87, 0x80, 0x80, 0x80, 0xC0, 0xC0, 0xE0, 0xF0, 0xF8, 0xFE, 0xFF
};

void InitMCU(){     //  Init MCU function
// SPI config
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_3);      // Set GPIOC pin 3 as digital output
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13);     // Set GPIOD pin 13 as digital output
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_12);     // Set GPIOC pin 12 as digital output
  GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_0);      // Set GPIOA pin 0 as digital output
  SPI3_Init_Advanced(_SPI_FPCLK_DIV8, _SPI_MASTER | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI3_PC10_11_12);
  Delay_ms(500);
}
//Send command to OLED m display
void OLED_M_command(unsigned char temp){
  OLED_CS=0;
  OLED_DC=0;
  SPI3_Write(temp);
  OLED_CS=1;
}
//Send data to OLED m display
void OLED_M_data(unsigned char temp){
  OLED_CS=0;
  OLED_DC=1;
  SPI3_Write(temp);
  OLED_CS=1;
}

void OLED_M_Init(){
                   OLED_RST=0;
                   Delay_ms(1000);
                   OLED_RST=1;
                   Delay_ms(1000);
                   OLED_M_command(SSD1306_DISPLAYOFF);             //0xAE  Set OLED Display Off
                   OLED_M_command(SSD1306_SETDISPLAYCLOCKDIV);     //0xD5  Set Display Clock Divide Ratio/Oscillator Frequency
                   OLED_M_command(0x80);
                   OLED_M_command(SSD1306_SETMULTIPLEX);           //0xA8  Set Multiplex Ratio
                   OLED_M_command(0x27);
                   OLED_M_command(SSD1306_SETDISPLAYOFFSET);       //0xD3  Set Display Offset
                   OLED_M_command(0x00);
                   OLED_M_command(SSD1306_SETSTARTLINE);           //0x40  Set Display Start Line
                   OLED_M_command(SSD1306_CHARGEPUMP);             //0x8D  Set Charge Pump
                   OLED_M_command(0x14);                           //0x14  Enable Charge Pump
                  // OLED_M_command(SSD1306_SETSTARTLINE | 0x0);     //A0
                   OLED_M_command(SSD1306_COMSCANDEC);             //0xC8  Set COM Output Scan Direction
                   OLED_M_command(SSD1306_SETCOMPINS);             //0xDA  Set COM Pins Hardware Configuration
                   OLED_M_command(0x12);
                   OLED_M_command(SSD1306_SETCONTRAST);            //0x81   Set Contrast Control
                   OLED_M_command(0xAF);
                   OLED_M_command(SSD1306_SETPRECHARGE);           //0xD9   Set Pre-Charge Period
                   OLED_M_command(0x25);
                   OLED_M_command(SSD1306_SETVCOMDETECT);          //0xDB   Set VCOMH Deselect Level
                   OLED_M_command(0x20);
                   OLED_M_command(SSD1306_DISPLAYALLON_RESUME);    //0xA4   Set Entire Display On/Off
                   OLED_M_command(SSD1306_NORMALDISPLAY);          //0xA6   Set Normal/Inverse Display
                   OLED_M_command(SSD1306_DISPLAYON);              //0xAF   Set OLED Display On
}

//Set page adress for Page Addressing Mode
void Set_Page_Address(unsigned char add)
{
    add=0xb0|add;
    OLED_M_command(add);
}
//Set column adress for Page Addressing Mode
void Set_Column_Address(unsigned char add)
{
        OLED_M_command((0x10|(add>>4)));
        OLED_M_command((0x0f&add));
}
//Display picture for Page Addressing Mode
void Display_Picture(unsigned char pic[])
{
    unsigned char i,j,num=0;
        for(i=0;i<0x05;i++)
        {
        Set_Page_Address(i);
   // Set_Column_Address(0x00);
       OLED_M_command(0x10);
                OLED_M_command(0x40);
        for(j=0;j<0x60;j++)
                {
                    OLED_M_data(pic[i*0x60+j]);
                }
        }
}
//Set contrast control
void contrast_control(unsigned char temp){
        OLED_M_command(SSD1306_SETCONTRAST);           //0x81   Set Contrast Control
        OLED_M_command(temp);                          // contrast step 1 to 256
}

void startscroll_Right(unsigned char x, unsigned char y){
       OLED_M_command(SSD1306_RIGHT_HORIZONTAL_SCROLL);      //0x26  Right Horizontal scroll
       OLED_M_command(0X00);                                 //dummy byte
       OLED_M_command(x);                                    //define start page address
       OLED_M_command(0X00);                                 //Set time interval between each scroll
       OLED_M_command(y);                                    //Define end page address
       OLED_M_command(0X00);                                 //dummy byte
       OLED_M_command(0XFF);                                 //dummy byte
       OLED_M_command(SSD1306_ACTIVATE_SCROLL);
}

void startscroll_Left(unsigned char x, unsigned char y){
       OLED_M_command(SSD1306_LEFT_HORIZONTAL_SCROLL);       //0x27  Right Horizontal scroll
       OLED_M_command(0X00);                                 //dummy byte
       OLED_M_command(x);                                    //define start page address
       OLED_M_command(0X00);                                 //Set time interval between each scroll
       OLED_M_command(y);                                    //Define end page address
       OLED_M_command(0X00);                                 //dummy byte
       OLED_M_command(0XFF);                                 //dummy byte
       OLED_M_command(SSD1306_ACTIVATE_SCROLL);              //0x2F Activate scroll
}

void startscroll_DiagRight(unsigned char x, unsigned char y){
       OLED_M_command(SSD1306_SET_VERTICAL_SCROLL_AREA);              //0xA3 Set Vertical Scroll Area
       OLED_M_command(0X00);                                          //Set No. of rows in top fixed area
       OLED_M_command(SSD1306_LCDHEIGHT);                             //Set No. of rows in scroll area
       OLED_M_command(SSD1306_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL);  //0x29 Vertical and Right Horizontal Scroll
       OLED_M_command(0X00);                                          //dummy byte
       OLED_M_command(x);                                             //Define start page address
       OLED_M_command(0X00);                                          //Set time interval between each scroll
       OLED_M_command(y);                                             //Define end page address
       OLED_M_command(0X01);                                          //Vertical scrolling offset
       OLED_M_command(SSD1306_ACTIVATE_SCROLL);                       //0x2F Activate scroll
}

void startscroll_DiagLeft(unsigned char x, unsigned char y){
       OLED_M_command(SSD1306_SET_VERTICAL_SCROLL_AREA);              //0xA3 Set Vertical Scroll Area
       OLED_M_command(0X00);                                          //Set No. of rows in top fixed area
       OLED_M_command(SSD1306_LCDHEIGHT);                             //Set No. of rows in scroll area
       OLED_M_command(SSD1306_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL);   //0x2A Vertical and Right Horizontal Scroll
       OLED_M_command(0X00);                                          //dummy byte
       OLED_M_command(x);                                             //Define start page address
       OLED_M_command(0X00);                                          //Set time interval between each scroll
       OLED_M_command(y);                                             //Define end page address
       OLED_M_command(0X01);                                          //Vertical scrolling offset
       OLED_M_command(SSD1306_ACTIVATE_SCROLL);                       //2F Activate scroll
}

void scroll_STOP(){
     OLED_M_command(0x2E);    //0x2E deactivate scroll
}

void main(){
      int i=0x00;
      InitMCU();
      OLED_M_Init();
      while(1){

               Display_Picture(pic);
               Delay_ms(2000);
               OLED_M_command(SSD1306_INVERTDISPLAY);
               Delay_ms(2000);
               OLED_M_command(SSD1306_NORMALDISPLAY);
               Delay_ms(2000);

               for(i=0xAF; i>0x00; i--){
                                        contrast_control(i);
                                        Delay_ms(20);
               }
               for(i=0x00; i<0xAF; i++){
                                        contrast_control(i);
                                        Delay_ms(20);
               }
               startscroll_Right(0x00, 0x05);
               Delay_ms(3000);
               scroll_STOP();
               Display_Picture(pic);
               
               
               startscroll_Left(0x00, 0x05);
               Delay_ms(3000);
               scroll_STOP();
               Display_Picture(pic);
               
               startscroll_DiagRight(0x00, 0x05);
               Delay_ms(3000);
               scroll_STOP();
               Display_Picture(pic);

               startscroll_DiagLeft(0x00, 0x05);
               Delay_ms(3000);
               scroll_STOP();
      }
}